/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef __COMPONENTS_H__
#define __COMPONENTS_H__

typedef struct _xml_dtc_caller xml_dtc_caller;

typedef struct _xml_dtc_component xml_dtc_component;

#include <resource.h>
#include <request.h>
#include <xml-dtc.h>
#include <libxml/tree.h>

struct _xml_dtc_component {
  char *name;
  char *type;
  int (* handle) (xml_dtc_caller *, xml_dtc_request *);
  int links;
  xml_dtc_resource *resource;
  int (* free_private) (void *);
  void *_private;
  xml_dtc_list *config;
};

struct _xml_dtc_caller {
  void *_private;
  char *type;
  char *name;
  int links;
  xml_dtc_resource *resource;
  int resolved;
  union {
    char *name;
    xml_dtc_component *object;
    int (*handle) (xml_dtc_caller *, xml_dtc_request *);
  } component;
  int (* hook_before) (xml_dtc_caller *, xml_dtc_request *);
  int (* hook_after) (xml_dtc_caller *, xml_dtc_request *);
  int (* free_private) (void *);
  xml_dtc_hash *params;
};

typedef struct _xml_dtc_caller_def xml_dtc_caller_def;

struct _xml_dtc_caller_def {
  const char *name;
  const char *component;
  xml_dtc_component * (*callback) (xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node);
};

typedef struct _xml_dtc_component_def xml_dtc_component_def;

struct _xml_dtc_component_def {
  const char *name;
  const char *type;
  int (*handle) (xml_dtc_caller *, xml_dtc_request *);
  xml_dtc_component * (*callback) (xml_dtc_handler *handler, xml_dtc_hash *handlers, xml_dtc_sitemap *sitemap, xmlNodePtr node);
};

xml_dtc_component *xml_dtc_component_new(const xml_dtc_resource *resource, const char *name, const char *type);

xml_dtc_component *xml_dtc_component_link(xml_dtc_component *component);

int xml_dtc_component_unlink(xml_dtc_component *component);

xml_dtc_caller *xml_dtc_caller_new(const xml_dtc_resource *resource, const char *name, const char *type);

int xml_dtc_caller_unlink(xml_dtc_caller *caller);

#endif /* __COMPONENTS_H__ */
