/*
 * libxml-dtc provides an XML framework creation facilities
 * Copyright (C) 2003-2004 Mike Hommey <mh@glandium.org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef __LIST_H__
#define __LIST_H__

typedef struct _xml_dtc_list xml_dtc_list;

#include <components.h>
#include <resource.h>

typedef void *(*xml_dtc_list_link_element_callback)(void *element);
typedef void (*xml_dtc_list_unlink_element_callback)(void *element);

struct _xml_dtc_list {
  xml_dtc_resource *resource;
  int len;
  xml_dtc_list_link_element_callback link_element;
  xml_dtc_list_unlink_element_callback unlink_element;
  void **list;
};

int xml_dtc_list_free(xml_dtc_list *list);

xml_dtc_list *xml_dtc_list_new(const xml_dtc_resource *resource, xml_dtc_list_unlink_element_callback unlink, xml_dtc_list_link_element_callback link);

#define xml_dtc_list_new_cast(resource, unlink, link) xml_dtc_list_new(resource, (xml_dtc_list_unlink_element_callback) (unlink), (xml_dtc_list_link_element_callback) (link))

#define xml_dtc_component_list_new(resource) xml_dtc_list_new_cast(resource, xml_dtc_component_unlink, xml_dtc_component_link)

#define xml_dtc_caller_list_new(resource) xml_dtc_list_new_cast(resource, xml_dtc_caller_unlink, NULL)

#define xml_dtc_string_list_new(resource) xml_dtc_list_new_cast(resource, free, strdup)

#define xml_dtc_value_list_new(resource) xml_dtc_list_new_cast(resource, NULL, NULL)

void *xml_dtc_list_get(xml_dtc_list *list, int index);

int xml_dtc_list_push(xml_dtc_list *list, void *element);

#define xml_dtc_list_push_cast(list, element) xml_dtc_list_push(list, (void *) (element))

void *xml_dtc_list_pop(xml_dtc_list *list);

#endif /* __LIST_H__ */
